@@ -24,7 +24,7 @@ use constant REPORT            => $ENV{JSON_VALIDATOR_REPORT} // DEBUG >= 2;
 use constant RECURSION_LIMIT   => $ENV{JSON_VALIDATOR_RECURSION_LIMIT} || 100;
 use constant SPECIFICATION_URL => 'http://json-schema.org/draft-04/schema#';
 
-our $VERSION = '3.03';
+our $VERSION = '3.04';
 our @EXPORT_OK = qw(joi validate_json);
 
 my $BUNDLED_CACHE_DIR = path(path(__FILE__)->dirname, qw(Validator cache));
@@ -195,14 +195,25 @@ sub validate_json {
 
 sub _build_formats {
   return {
+    'date'          => JSON::Validator::Formats->can('check_date'),
     'date-time'     => JSON::Validator::Formats->can('check_date_time'),
     'email'         => JSON::Validator::Formats->can('check_email'),
     'hostname'      => JSON::Validator::Formats->can('check_hostname'),
+    'idn-email'     => JSON::Validator::Formats->can('check_idn_email'),
+    'idn-hostname'  => JSON::Validator::Formats->can('check_idn_hostname'),
     'ipv4'          => JSON::Validator::Formats->can('check_ipv4'),
     'ipv6'          => JSON::Validator::Formats->can('check_ipv6'),
+    'iri'           => JSON::Validator::Formats->can('check_iri'),
+    'iri-reference' => JSON::Validator::Formats->can('check_iri_reference'),
+    'json-pointer'  => JSON::Validator::Formats->can('check_json_pointer'),
     'regex'         => JSON::Validator::Formats->can('check_regex'),
+    'relative-json-pointer' =>
+      JSON::Validator::Formats->can('check_relative_json_pointer'),
+    'time'          => JSON::Validator::Formats->can('check_time'),
     'uri'           => JSON::Validator::Formats->can('check_uri'),
     'uri-reference' => JSON::Validator::Formats->can('check_uri_reference'),
+    'uri-reference' => JSON::Validator::Formats->can('check_uri_reference'),
+    'uri-template'  => JSON::Validator::Formats->can('check_uri_template'),
   };
 }
 
@@ -520,6 +531,9 @@ sub _validate {
   my ($self, $data, $path, $schema) = @_;
   my ($seen_addr, $to_json, $type);
 
+  # Do not validate against "default" in draft-07 schema
+  return if blessed $schema and $schema->isa('JSON::PP::Boolean');
+
   $schema = $self->_ref_to_schema($schema) if $schema->{'$ref'};
   $seen_addr = join ':', refaddr($schema),
     (!defined $data ? 'c:undef' : ref $data ? refaddr $data : "s:$data");
@@ -724,7 +738,16 @@ sub _validate_type_array {
       last;
     }
   }
-  if (ref $schema->{items} eq 'ARRAY') {
+
+  if ($schema->{contains}) {
+    my @e;
+    for my $i (0 .. @$data - 1) {
+      my @tmp = $self->_validate($data->[$i], "$path/$i", $schema->{contains});
+      push @e, \@tmp if @tmp;
+    }
+    push @errors, map {@$_} @e if @e >= @$data;
+  }
+  elsif (ref $schema->{items} eq 'ARRAY') {
     my $additional_items = $schema->{additionalItems} // {type => 'any'};
     my @rules = @{$schema->{items}};
 
@@ -848,6 +871,19 @@ sub _validate_type_object {
     push @errors, E $path, sprintf 'Not enough properties: %s/%s.', int @dkeys,
       $schema->{minProperties};
   }
+  if (my $n_schema = $schema->{propertyNames}) {
+    for my $name (keys %$data) {
+      next unless my @e = $self->_validate($name, $path, $n_schema);
+      push @errors,
+        _add_path_to_error_messages(propertyName => [map { ($name, $_) } @e]);
+    }
+  }
+  if ($schema->{if}) {
+    push @errors,
+      $self->_validate($data, $path, $schema->{if})
+      ? $self->_validate($data, $path, $schema->{else} // {})
+      : $self->_validate($data, $path, $schema->{then} // {});
+  }
 
   while (my ($k, $r) = each %{$schema->{properties}}) {
     push @{$rules{$k}}, $r;
@@ -880,12 +916,13 @@ sub _validate_type_object {
       next unless exists $data->{$k};
       my @e = $self->_validate($data->{$k}, _path($path, $k), $r);
       push @errors, @e;
+      next if @e or !UNIVERSAL::isa($r, 'HASH');
       push @errors,
         $self->_validate_type_enum($data->{$k}, _path($path, $k), $r)
-        if $r->{enum} and !@e;
+        if $r->{enum};
       push @errors,
         $self->_validate_type_const($data->{$k}, _path($path, $k), $r)
-        if $r->{const} and !@e;
+        if $r->{const};
     }
   }
 
@@ -986,7 +1023,9 @@ sub _guess_schema_type {
   return _guessed_right(object => $_[1]) if $_[0]->{additionalProperties};
   return _guessed_right(object => $_[1]) if $_[0]->{patternProperties};
   return _guessed_right(object => $_[1]) if $_[0]->{properties};
+  return _guessed_right(object => $_[1]) if $_[0]->{propertyNames};
   return _guessed_right(object => $_[1]) if $_[0]->{required};
+  return _guessed_right(object => $_[1]) if $_[0]->{if};
   return _guessed_right(object => $_[1])
     if defined $_[0]->{maxProperties}
     or defined $_[0]->{minProperties};
